/*******************************************************************************
* File Name: MAX7219.c
* Version 2.0
*
* Description:
*  This file provides the source code to the API for the SCB Component.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "MAX7219_PVT.h"

#if(MAX7219_SCB_MODE_I2C_INC)
    #include "MAX7219_I2C_PVT.h"
#endif /* (MAX7219_SCB_MODE_I2C_INC) */

#if(MAX7219_SCB_MODE_EZI2C_INC)
    #include "MAX7219_EZI2C_PVT.h"
#endif /* (MAX7219_SCB_MODE_EZI2C_INC) */

#if(MAX7219_SCB_MODE_SPI_INC || MAX7219_SCB_MODE_UART_INC)
    #include "MAX7219_SPI_UART_PVT.h"
#endif /* (MAX7219_SCB_MODE_SPI_INC || MAX7219_SCB_MODE_UART_INC) */


/***************************************
*    Run Time Configuration Vars
***************************************/

/* Stores internal component configuration for Unconfigured mode */
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Common configuration variables */
    uint8 MAX7219_scbMode = MAX7219_SCB_MODE_UNCONFIG;
    uint8 MAX7219_scbEnableWake;
    uint8 MAX7219_scbEnableIntr;

    /* I2C configuration variables */
    uint8 MAX7219_mode;
    uint8 MAX7219_acceptAddr;

    /* SPI/UART configuration variables */
    volatile uint8 * MAX7219_rxBuffer;
    uint8  MAX7219_rxDataBits;
    uint32 MAX7219_rxBufferSize;

    volatile uint8 * MAX7219_txBuffer;
    uint8  MAX7219_txDataBits;
    uint32 MAX7219_txBufferSize;

    /* EZI2C configuration variables */
    uint8 MAX7219_numberOfAddr;
    uint8 MAX7219_subAddrSize;
#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */


/***************************************
*     Common SCB Vars
***************************************/

uint8 MAX7219_initVar = 0u;

#if !defined (CY_REMOVE_MAX7219_CUSTOM_INTR_HANDLER)
    cyisraddress MAX7219_customIntrHandler = NULL;
#endif /* !defined (CY_REMOVE_MAX7219_CUSTOM_INTR_HANDLER) */


/***************************************
*    Private Function Prototypes
***************************************/

static void MAX7219_ScbEnableIntr(void);
static void MAX7219_ScbModeStop(void);


/*******************************************************************************
* Function Name: MAX7219_Init
********************************************************************************
*
* Summary:
*  Initializes the SCB component to operate in one of the selected
*  configurations: I2C, SPI, UART or EZI2C.
*  When the configuration is set to "Unconfigured SCB", this function does
*  not do any initialization. Use mode-specific initialization APIs instead:
*  SCB_I2CInit, SCB_SpiInit, SCB_UartInit or SCB_EzI2CInit.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_Init(void)
{
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    if(MAX7219_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        MAX7219_initVar = 0u;
    }
    else
    {
        /* Initialization was done before this function call */
    }

#elif(MAX7219_SCB_MODE_I2C_CONST_CFG)
    MAX7219_I2CInit();

#elif(MAX7219_SCB_MODE_SPI_CONST_CFG)
    MAX7219_SpiInit();

#elif(MAX7219_SCB_MODE_UART_CONST_CFG)
    MAX7219_UartInit();

#elif(MAX7219_SCB_MODE_EZI2C_CONST_CFG)
    MAX7219_EzI2CInit();

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: MAX7219_Enable
********************************************************************************
*
* Summary:
*  Enables the SCB component operation.
*  The SCB configuration should be not changed when the component is enabled.
*  Any configuration changes should be made after disabling the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_Enable(void)
{
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    /* Enable SCB block, only if it is already configured */
    if(!MAX7219_SCB_MODE_UNCONFIG_RUNTM_CFG)
    {
        MAX7219_CTRL_REG |= MAX7219_CTRL_ENABLED;

        MAX7219_ScbEnableIntr();
    }
#else
    MAX7219_CTRL_REG |= MAX7219_CTRL_ENABLED;

    MAX7219_ScbEnableIntr();
#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: MAX7219_Start
********************************************************************************
*
* Summary:
*  Invokes SCB_Init() and SCB_Enable().
*  After this function call, the component is enabled and ready for operation.
*  When configuration is set to "Unconfigured SCB", the component must first be
*  initialized to operate in one of the following configurations: I2C, SPI, UART
*  or EZ I2C. Otherwise this function does not enable the component.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*  MAX7219_initVar - used to check initial configuration, modified
*  on first function call.
*
*******************************************************************************/
void MAX7219_Start(void)
{
    if(0u == MAX7219_initVar)
    {
        MAX7219_Init();
        MAX7219_initVar = 1u; /* Component was initialized */
    }

    MAX7219_Enable();
}


/*******************************************************************************
* Function Name: MAX7219_Stop
********************************************************************************
*
* Summary:
*  Disables the SCB component and its interrupt.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_Stop(void)
{
#if(MAX7219_SCB_IRQ_INTERNAL)
    MAX7219_DisableInt();
#endif /* (MAX7219_SCB_IRQ_INTERNAL) */

    MAX7219_CTRL_REG &= (uint32) ~MAX7219_CTRL_ENABLED;  /* Disable scb IP */

#if(MAX7219_SCB_IRQ_INTERNAL)
    MAX7219_ClearPendingInt();
#endif /* (MAX7219_SCB_IRQ_INTERNAL) */

    MAX7219_ScbModeStop(); /* Calls scbMode specific Stop function */
}


/*******************************************************************************
* Function Name: MAX7219_SetRxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the RX FIFO to generate a RX level interrupt.
*  When the RX FIFO has more entries than the RX FIFO level an RX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the RX FIFO to generate RX level interrupt.
*         The range of valid level values is between 0 and RX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_SetRxFifoLevel(uint32 level)
{
    uint32 rxFifoCtrl;

    rxFifoCtrl = MAX7219_RX_FIFO_CTRL_REG;

    rxFifoCtrl &= ((uint32) ~MAX7219_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    rxFifoCtrl |= ((uint32) (MAX7219_RX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    MAX7219_RX_FIFO_CTRL_REG = rxFifoCtrl;
}


/*******************************************************************************
* Function Name: MAX7219_SetTxFifoLevel
********************************************************************************
*
* Summary:
*  Sets level in the TX FIFO to generate a TX level interrupt.
*  When the TX FIFO has more entries than the TX FIFO level an TX level
*  interrupt request is generated.
*
* Parameters:
*  level: Level in the TX FIFO to generate TX level interrupt.
*         The range of valid level values is between 0 and TX FIFO depth - 1.
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_SetTxFifoLevel(uint32 level)
{
    uint32 txFifoCtrl;

    txFifoCtrl = MAX7219_TX_FIFO_CTRL_REG;

    txFifoCtrl &= ((uint32) ~MAX7219_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK); /* Clear level mask bits */
    txFifoCtrl |= ((uint32) (MAX7219_TX_FIFO_CTRL_TRIGGER_LEVEL_MASK & level));

    MAX7219_TX_FIFO_CTRL_REG = txFifoCtrl;
}


/*******************************************************************************
* Function Name: MAX7219_SetCustomInterruptHandler
********************************************************************************
*
* Summary:
*  Registers a function to be called by the internal interrupt handler.
*  First the function that is registered is called, then the internal interrupt
*  handler performs any operation such as software buffer management functions
*  before the interrupt returns.  It is the user's responsibility not to break
*  the software buffer operations. Only one custom handler is supported, which
*  is the function provided by the most recent call.
*  At the initialization time no custom handler is registered.
*
* Parameters:
*  func: Pointer to the function to register.
*        The value NULL indicates to remove the current custom interrupt
*        handler.
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_SetCustomInterruptHandler(cyisraddress func)
{
#if !defined (CY_REMOVE_MAX7219_CUSTOM_INTR_HANDLER)
    MAX7219_customIntrHandler = func; /* Register interrupt handler */
#else
    if(NULL != func)
    {
        /* Suppress compiler warning */
    }
#endif /* !defined (CY_REMOVE_MAX7219_CUSTOM_INTR_HANDLER) */
}


/*******************************************************************************
* Function Name: MAX7219_ScbModeEnableIntr
********************************************************************************
*
* Summary:
*  Enables an interrupt for a specific mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void MAX7219_ScbEnableIntr(void)
{
#if(MAX7219_SCB_IRQ_INTERNAL)
    #if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
        /* Enable interrupt in NVIC */
        if(0u != MAX7219_scbEnableIntr)
        {
            MAX7219_EnableInt();
        }
    #else
        MAX7219_EnableInt();

    #endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
#endif /* (MAX7219_SCB_IRQ_INTERNAL) */
}


/*******************************************************************************
* Function Name: MAX7219_ScbModeStop
********************************************************************************
*
* Summary:
*  Calls the Stop function for a specific operation mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
static void MAX7219_ScbModeStop(void)
{
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    if(MAX7219_SCB_MODE_I2C_RUNTM_CFG)
    {
        MAX7219_I2CStop();
    }
    else if (MAX7219_SCB_MODE_EZI2C_RUNTM_CFG)
    {
        MAX7219_EzI2CStop();
    }
#if (!MAX7219_CY_SCBIP_V1)
    else if (MAX7219_SCB_MODE_UART_RUNTM_CFG)
    {
        MAX7219_UartStop();
    }
#endif /* (!MAX7219_CY_SCBIP_V1) */
    else
    {
        /* Do nothing for other modes */
    }
#elif(MAX7219_SCB_MODE_I2C_CONST_CFG)
    MAX7219_I2CStop();

#elif(MAX7219_SCB_MODE_EZI2C_CONST_CFG)
    MAX7219_EzI2CStop();

#elif(MAX7219_SCB_MODE_UART_CONST_CFG)
    MAX7219_UartStop();

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
}


#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: MAX7219_SetPins
    ********************************************************************************
    *
    * Summary:
    *  Sets the pins settings accordingly to the selected operation mode.
    *  Only available in the Unconfigured operation mode. The mode specific
    *  initialization function calls it.
    *  Pins configuration is set by PSoC Creator when a specific mode of operation
    *  is selected in design time.
    *
    * Parameters:
    *  mode:      Mode of SCB operation.
    *  subMode:   Sub-mode of SCB operation. It is only required for SPI and UART
    *             modes.
    *  uartEnableMask: enables TX or RX direction and RTS and CTS signals.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_SetPins(uint32 mode, uint32 subMode, uint32 uartEnableMask)
    {
        uint32 hsiomSel [MAX7219_SCB_PINS_NUMBER];
        uint32 pinsDm   [MAX7219_SCB_PINS_NUMBER];

    #if (!MAX7219_CY_SCBIP_V1)
        uint32 pinsInBuf = 0u;
    #endif /* (!MAX7219_CY_SCBIP_V1) */

        uint32 i;

        /* Set default HSIOM to GPIO and Drive Mode to Analog Hi-Z */
        for(i = 0u; i < MAX7219_SCB_PINS_NUMBER; i++)
        {
            hsiomSel[i]  = MAX7219_HSIOM_DEF_SEL;
            pinsDm[i]    = MAX7219_PIN_DM_ALG_HIZ;
        }

        if((MAX7219_SCB_MODE_I2C   == mode) ||
           (MAX7219_SCB_MODE_EZI2C == mode))
        {
            hsiomSel[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_HSIOM_I2C_SEL;
            hsiomSel[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_HSIOM_I2C_SEL;

            pinsDm[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_PIN_DM_OD_LO;
            pinsDm[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_PIN_DM_OD_LO;
        }
    #if (!MAX7219_CY_SCBIP_V1)
        else if(MAX7219_SCB_MODE_SPI == mode)
        {
            hsiomSel[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
            hsiomSel[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
            hsiomSel[MAX7219_SCLK_PIN_INDEX]        = MAX7219_HSIOM_SPI_SEL;

            if(MAX7219_SPI_SLAVE == subMode)
            {
                /* Slave */
                pinsDm[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_PIN_DM_DIG_HIZ;
                pinsDm[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsDm[MAX7219_SCLK_PIN_INDEX]        = MAX7219_PIN_DM_DIG_HIZ;

            #if(MAX7219_SS0_PIN)
                /* Only SS0 is valid choice for Slave */
                hsiomSel[MAX7219_SS0_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
                pinsDm  [MAX7219_SS0_PIN_INDEX] = MAX7219_PIN_DM_DIG_HIZ;
            #endif /* (MAX7219_SS1_PIN) */

            #if(MAX7219_MISO_SDA_TX_PIN)
                /* Disable input buffer */
                 pinsInBuf |= MAX7219_MISO_SDA_TX_PIN_MASK;
            #endif /* (MAX7219_MISO_SDA_TX_PIN_PIN) */
            }
            else /* (Master) */
            {
                pinsDm[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsDm[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_PIN_DM_DIG_HIZ;
                pinsDm[MAX7219_SCLK_PIN_INDEX]        = MAX7219_PIN_DM_STRONG;

            #if(MAX7219_SS0_PIN)
                hsiomSel [MAX7219_SS0_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
                pinsDm   [MAX7219_SS0_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsInBuf                                |= MAX7219_SS0_PIN_MASK;
            #endif /* (MAX7219_SS0_PIN) */

            #if(MAX7219_SS1_PIN)
                hsiomSel [MAX7219_SS1_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
                pinsDm   [MAX7219_SS1_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsInBuf                                |= MAX7219_SS1_PIN_MASK;
            #endif /* (MAX7219_SS1_PIN) */

            #if(MAX7219_SS2_PIN)
                hsiomSel [MAX7219_SS2_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
                pinsDm   [MAX7219_SS2_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsInBuf                                |= MAX7219_SS2_PIN_MASK;
            #endif /* (MAX7219_SS2_PIN) */

            #if(MAX7219_SS3_PIN)
                hsiomSel [MAX7219_SS3_PIN_INDEX] = MAX7219_HSIOM_SPI_SEL;
                pinsDm   [MAX7219_SS3_PIN_INDEX] = MAX7219_PIN_DM_STRONG;
                pinsInBuf                                |= MAX7219_SS3_PIN_MASK;
            #endif /* (MAX7219_SS2_PIN) */

                /* Disable input buffers */
            #if(MAX7219_MOSI_SCL_RX_PIN)
                pinsInBuf |= MAX7219_MOSI_SCL_RX_PIN_MASK;
            #endif /* (MAX7219_MOSI_SCL_RX_PIN) */

             #if(MAX7219_MOSI_SCL_RX_WAKE_PIN)
                pinsInBuf |= MAX7219_MOSI_SCL_RX_WAKE_PIN_MASK;
            #endif /* (MAX7219_MOSI_SCL_RX_WAKE_PIN) */

            #if(MAX7219_SCLK_PIN)
                pinsInBuf |= MAX7219_SCLK_PIN_MASK;
            #endif /* (MAX7219_SCLK_PIN) */
            }
        }
        else /* UART */
        {
            if(MAX7219_UART_MODE_SMARTCARD == subMode)
            {
                /* SmartCard */
                hsiomSel[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_HSIOM_UART_SEL;
                pinsDm  [MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_PIN_DM_OD_LO;
            }
            else /* Standard or IrDA */
            {
                if(0u != (MAX7219_UART_RX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_HSIOM_UART_SEL;
                    pinsDm  [MAX7219_MOSI_SCL_RX_PIN_INDEX] = MAX7219_PIN_DM_DIG_HIZ;
                }

                if(0u != (MAX7219_UART_TX_PIN_ENABLE & uartEnableMask))
                {
                    hsiomSel[MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_HSIOM_UART_SEL;
                    pinsDm  [MAX7219_MISO_SDA_TX_PIN_INDEX] = MAX7219_PIN_DM_STRONG;

                #if(MAX7219_MISO_SDA_TX_PIN)
                     pinsInBuf |= MAX7219_MISO_SDA_TX_PIN_MASK;
                #endif /* (MAX7219_MISO_SDA_TX_PIN_PIN) */
                }

            #if !(MAX7219_CY_SCBIP_V0 || MAX7219_CY_SCBIP_V1)
                if(MAX7219_UART_MODE_STD == subMode)
                {
                    if(0u != (MAX7219_UART_CTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* CTS input is multiplexed with SCLK */
                        hsiomSel[MAX7219_SCLK_PIN_INDEX] = MAX7219_HSIOM_UART_SEL;
                        pinsDm  [MAX7219_SCLK_PIN_INDEX] = MAX7219_PIN_DM_DIG_HIZ;
                    }

                    if(0u != (MAX7219_UART_RTS_PIN_ENABLE & uartEnableMask))
                    {
                        /* RTS output is multiplexed with SS0 */
                        hsiomSel[MAX7219_SS0_PIN_INDEX] = MAX7219_HSIOM_UART_SEL;
                        pinsDm  [MAX7219_SS0_PIN_INDEX] = MAX7219_PIN_DM_STRONG;

                    #if(MAX7219_SS0_PIN)
                        /* Disable input buffer */
                        pinsInBuf |= MAX7219_SS0_PIN_MASK;
                    #endif /* (MAX7219_SS0_PIN) */
                    }
                }
            #endif /* !(MAX7219_CY_SCBIP_V0 || MAX7219_CY_SCBIP_V1) */
            }
        }
    #endif /* (!MAX7219_CY_SCBIP_V1) */

    /* Configure pins: set HSIOM, DM and InputBufEnable */
    /* Note: the DR register settings do not effect the pin output if HSIOM is other than GPIO */

    #if(MAX7219_MOSI_SCL_RX_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_MOSI_SCL_RX_HSIOM_REG,
                                       MAX7219_MOSI_SCL_RX_HSIOM_MASK,
                                       MAX7219_MOSI_SCL_RX_HSIOM_POS,
                                       hsiomSel[MAX7219_MOSI_SCL_RX_PIN_INDEX]);

        MAX7219_spi_mosi_i2c_sda_uart_rx_SetDriveMode((uint8) pinsDm[MAX7219_MOSI_SCL_RX_PIN_INDEX]);

    #if (!MAX7219_CY_SCBIP_V1)
        MAX7219_SET_INP_DIS(MAX7219_spi_mosi_i2c_sda_uart_rx_INP_DIS,
                                     MAX7219_spi_mosi_i2c_sda_uart_rx_MASK,
                                     (0u != (pinsInBuf & MAX7219_MOSI_SCL_RX_PIN_MASK)));
    #endif /* (!MAX7219_CY_SCBIP_V1) */
    #endif /* (MAX7219_MOSI_SCL_RX_PIN) */

    #if(MAX7219_MOSI_SCL_RX_WAKE_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_MOSI_SCL_RX_WAKE_HSIOM_REG,
                                       MAX7219_MOSI_SCL_RX_WAKE_HSIOM_MASK,
                                       MAX7219_MOSI_SCL_RX_WAKE_HSIOM_POS,
                                       hsiomSel[MAX7219_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        MAX7219_spi_mosi_i2c_sda_uart_rx_wake_SetDriveMode((uint8)
                                                               pinsDm[MAX7219_MOSI_SCL_RX_WAKE_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_mosi_i2c_sda_uart_rx_wake_INP_DIS,
                                     MAX7219_spi_mosi_i2c_sda_uart_rx_wake_MASK,
                                     (0u != (pinsInBuf & MAX7219_MOSI_SCL_RX_WAKE_PIN_MASK)));

         /* Set interrupt on falling edge */
        MAX7219_SET_INCFG_TYPE(MAX7219_MOSI_SCL_RX_WAKE_INTCFG_REG,
                                        MAX7219_MOSI_SCL_RX_WAKE_INTCFG_TYPE_MASK,
                                        MAX7219_MOSI_SCL_RX_WAKE_INTCFG_TYPE_POS,
                                        MAX7219_INTCFG_TYPE_FALLING_EDGE);
    #endif /* (MAX7219_MOSI_SCL_RX_WAKE_PIN) */

    #if(MAX7219_MISO_SDA_TX_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_MISO_SDA_TX_HSIOM_REG,
                                       MAX7219_MISO_SDA_TX_HSIOM_MASK,
                                       MAX7219_MISO_SDA_TX_HSIOM_POS,
                                       hsiomSel[MAX7219_MISO_SDA_TX_PIN_INDEX]);

        MAX7219_spi_miso_i2c_scl_uart_tx_SetDriveMode((uint8) pinsDm[MAX7219_MISO_SDA_TX_PIN_INDEX]);

    #if (!MAX7219_CY_SCBIP_V1)
        MAX7219_SET_INP_DIS(MAX7219_spi_miso_i2c_scl_uart_tx_INP_DIS,
                                     MAX7219_spi_miso_i2c_scl_uart_tx_MASK,
                                    (0u != (pinsInBuf & MAX7219_MISO_SDA_TX_PIN_MASK)));
    #endif /* (!MAX7219_CY_SCBIP_V1) */
    #endif /* (MAX7219_MOSI_SCL_RX_PIN) */

    #if(MAX7219_SCLK_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_SCLK_HSIOM_REG, MAX7219_SCLK_HSIOM_MASK,
                                       MAX7219_SCLK_HSIOM_POS, hsiomSel[MAX7219_SCLK_PIN_INDEX]);

        MAX7219_spi_sclk_uart_cts_SetDriveMode((uint8) pinsDm[MAX7219_SCLK_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_sclk_uart_cts_INP_DIS,
                                     MAX7219_spi_sclk_uart_cts_MASK,
                                     (0u != (pinsInBuf & MAX7219_SCLK_PIN_MASK)));
    #endif /* (MAX7219_SCLK_PIN) */

    #if(MAX7219_SS0_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_SS0_HSIOM_REG, MAX7219_SS0_HSIOM_MASK,
                                       MAX7219_SS0_HSIOM_POS, hsiomSel[MAX7219_SS0_PIN_INDEX]);

        MAX7219_spi_ss0_uart_rts_SetDriveMode((uint8) pinsDm[MAX7219_SS0_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_ss0_uart_rts_INP_DIS,
                                     MAX7219_spi_ss0_uart_rts_MASK,
                                     (0u != (pinsInBuf & MAX7219_SS0_PIN_MASK)));
    #endif /* (MAX7219_SS1_PIN) */

    #if(MAX7219_SS1_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_SS1_HSIOM_REG, MAX7219_SS1_HSIOM_MASK,
                                       MAX7219_SS1_HSIOM_POS, hsiomSel[MAX7219_SS1_PIN_INDEX]);

        MAX7219_spi_ss1_SetDriveMode((uint8) pinsDm[MAX7219_SS1_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_ss1_INP_DIS,
                                     MAX7219_spi_ss1_MASK,
                                     (0u != (pinsInBuf & MAX7219_SS1_PIN_MASK)));
    #endif /* (MAX7219_SS1_PIN) */

    #if(MAX7219_SS2_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_SS2_HSIOM_REG, MAX7219_SS2_HSIOM_MASK,
                                       MAX7219_SS2_HSIOM_POS, hsiomSel[MAX7219_SS2_PIN_INDEX]);

        MAX7219_spi_ss2_SetDriveMode((uint8) pinsDm[MAX7219_SS2_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_ss2_INP_DIS,
                                     MAX7219_spi_ss2_MASK,
                                     (0u != (pinsInBuf & MAX7219_SS2_PIN_MASK)));
    #endif /* (MAX7219_SS2_PIN) */

    #if(MAX7219_SS3_PIN)
        MAX7219_SET_HSIOM_SEL(MAX7219_SS3_HSIOM_REG,  MAX7219_SS3_HSIOM_MASK,
                                       MAX7219_SS3_HSIOM_POS, hsiomSel[MAX7219_SS3_PIN_INDEX]);

        MAX7219_spi_ss3_SetDriveMode((uint8) pinsDm[MAX7219_SS3_PIN_INDEX]);

        MAX7219_SET_INP_DIS(MAX7219_spi_ss3_INP_DIS,
                                     MAX7219_spi_ss3_MASK,
                                     (0u != (pinsInBuf & MAX7219_SS3_PIN_MASK)));
    #endif /* (MAX7219_SS3_PIN) */
    }

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */


#if (MAX7219_CY_SCBIP_V0 || MAX7219_CY_SCBIP_V1)
    /*******************************************************************************
    * Function Name: MAX7219_I2CSlaveNackGeneration
    ********************************************************************************
    *
    * Summary:
    *  Sets command to generate NACK to the address or data.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_I2CSlaveNackGeneration(void)
    {
        /* Check for EC_AM toggle condition: EC_AM and clock stretching for address are enabled */
        if ((0u != (MAX7219_CTRL_REG & MAX7219_CTRL_EC_AM_MODE)) &&
            (0u == (MAX7219_I2C_CTRL_REG & MAX7219_I2C_CTRL_S_NOT_READY_ADDR_NACK)))
        {
            /* Toggle EC_AM before NACK generation */
            MAX7219_CTRL_REG &= ~MAX7219_CTRL_EC_AM_MODE;
            MAX7219_CTRL_REG |=  MAX7219_CTRL_EC_AM_MODE;
        }

        MAX7219_I2C_SLAVE_CMD_REG = MAX7219_I2C_SLAVE_CMD_S_NACK;
    }
#endif /* (MAX7219_CY_SCBIP_V0 || MAX7219_CY_SCBIP_V1) */


/* [] END OF FILE */
