/*******************************************************************************
* File Name: MAX7219_PM.c
* Version 2.0
*
* Description:
*  This file provides the source code to the Power Management support for
*  the SCB Component.
*
* Note:
*
********************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "MAX7219.h"
#include "MAX7219_PVT.h"

#if(MAX7219_SCB_MODE_I2C_INC)
    #include "MAX7219_I2C_PVT.h"
#endif /* (MAX7219_SCB_MODE_I2C_INC) */

#if(MAX7219_SCB_MODE_EZI2C_INC)
    #include "MAX7219_EZI2C_PVT.h"
#endif /* (MAX7219_SCB_MODE_EZI2C_INC) */

#if(MAX7219_SCB_MODE_SPI_INC || MAX7219_SCB_MODE_UART_INC)
    #include "MAX7219_SPI_UART_PVT.h"
#endif /* (MAX7219_SCB_MODE_SPI_INC || MAX7219_SCB_MODE_UART_INC) */


/***************************************
*   Backup Structure declaration
***************************************/

#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG || \
   (MAX7219_SCB_MODE_I2C_CONST_CFG   && (!MAX7219_I2C_WAKE_ENABLE_CONST))   || \
   (MAX7219_SCB_MODE_EZI2C_CONST_CFG && (!MAX7219_EZI2C_WAKE_ENABLE_CONST)) || \
   (MAX7219_SCB_MODE_SPI_CONST_CFG   && (!MAX7219_SPI_WAKE_ENABLE_CONST))   || \
   (MAX7219_SCB_MODE_UART_CONST_CFG  && (!MAX7219_UART_WAKE_ENABLE_CONST)))

    MAX7219_BACKUP_STRUCT MAX7219_backup =
    {
        0u, /* enableState */
    };
#endif


/*******************************************************************************
* Function Name: MAX7219_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component to enter Deep Sleep.
*  The "Enable wakeup from Sleep Mode" selection has an influence on
*  this function implementation.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_Sleep(void)
{
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)

    if(MAX7219_SCB_WAKE_ENABLE_CHECK)
    {
        if(MAX7219_SCB_MODE_I2C_RUNTM_CFG)
        {
            MAX7219_I2CSaveConfig();
        }
        else if(MAX7219_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            MAX7219_EzI2CSaveConfig();
        }
    #if(!MAX7219_CY_SCBIP_V1)
        else if(MAX7219_SCB_MODE_SPI_RUNTM_CFG)
        {
            MAX7219_SpiSaveConfig();
        }
        else if(MAX7219_SCB_MODE_UART_RUNTM_CFG)
        {
            MAX7219_UartSaveConfig();
        }
    #endif /* (!MAX7219_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        MAX7219_backup.enableState = (uint8) MAX7219_GET_CTRL_ENABLED;

        if(0u != MAX7219_backup.enableState)
        {
            MAX7219_Stop();
        }
    }

#else

    #if (MAX7219_SCB_MODE_I2C_CONST_CFG && MAX7219_I2C_WAKE_ENABLE_CONST)
        MAX7219_I2CSaveConfig();

    #elif (MAX7219_SCB_MODE_EZI2C_CONST_CFG && MAX7219_EZI2C_WAKE_ENABLE_CONST)
        MAX7219_EzI2CSaveConfig();

    #elif (MAX7219_SCB_MODE_SPI_CONST_CFG && MAX7219_SPI_WAKE_ENABLE_CONST)
        MAX7219_SpiSaveConfig();

    #elif (MAX7219_SCB_MODE_UART_CONST_CFG && MAX7219_UART_WAKE_ENABLE_CONST)
        MAX7219_UartSaveConfig();

    #else

        MAX7219_backup.enableState = (uint8) MAX7219_GET_CTRL_ENABLED;

        if(0u != MAX7219_backup.enableState)
        {
            MAX7219_Stop();
        }

    #endif /* defined (MAX7219_SCB_MODE_I2C_CONST_CFG) && (MAX7219_I2C_WAKE_ENABLE_CONST) */

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/*******************************************************************************
* Function Name: MAX7219_Wakeup
********************************************************************************
*
* Summary:
*  Prepares the component for the Active mode operation after exiting
*  Deep Sleep. The "Enable wakeup from Sleep Mode" option has an influence
*  on this function implementation.
*  This function should not be called after exiting Sleep.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void MAX7219_Wakeup(void)
{
#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)

    if(MAX7219_SCB_WAKE_ENABLE_CHECK)
    {
        if(MAX7219_SCB_MODE_I2C_RUNTM_CFG)
        {
            MAX7219_I2CRestoreConfig();
        }
        else if(MAX7219_SCB_MODE_EZI2C_RUNTM_CFG)
        {
            MAX7219_EzI2CRestoreConfig();
        }
    #if(!MAX7219_CY_SCBIP_V1)
        else if(MAX7219_SCB_MODE_SPI_RUNTM_CFG)
        {
            MAX7219_SpiRestoreConfig();
        }
        else if(MAX7219_SCB_MODE_UART_RUNTM_CFG)
        {
            MAX7219_UartRestoreConfig();
        }
    #endif /* (!MAX7219_CY_SCBIP_V1) */
        else
        {
            /* Unknown mode */
        }
    }
    else
    {
        if(0u != MAX7219_backup.enableState)
        {
            MAX7219_Enable();
        }
    }

#else

    #if (MAX7219_SCB_MODE_I2C_CONST_CFG  && MAX7219_I2C_WAKE_ENABLE_CONST)
        MAX7219_I2CRestoreConfig();

    #elif (MAX7219_SCB_MODE_EZI2C_CONST_CFG && MAX7219_EZI2C_WAKE_ENABLE_CONST)
        MAX7219_EzI2CRestoreConfig();

    #elif (MAX7219_SCB_MODE_SPI_CONST_CFG && MAX7219_SPI_WAKE_ENABLE_CONST)
        MAX7219_SpiRestoreConfig();

    #elif (MAX7219_SCB_MODE_UART_CONST_CFG && MAX7219_UART_WAKE_ENABLE_CONST)
        MAX7219_UartRestoreConfig();

    #else

        if(0u != MAX7219_backup.enableState)
        {
            MAX7219_Enable();
        }

    #endif /* (MAX7219_I2C_WAKE_ENABLE_CONST) */

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */
}


/* [] END OF FILE */
