/*******************************************************************************
* File Name: MAX7219_SPI_UART.c
* Version 2.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  SPI and UART modes.
*
* Note:
*
*******************************************************************************
* Copyright 2013-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "MAX7219_PVT.h"
#include "MAX7219_SPI_UART_PVT.h"

/***************************************
*        SPI/UART Private Vars
***************************************/

#if(MAX7219_INTERNAL_RX_SW_BUFFER_CONST)
    volatile uint32 MAX7219_rxBufferHead;
    volatile uint32 MAX7219_rxBufferTail;
    volatile uint8  MAX7219_rxBufferOverflow;
#endif /* (MAX7219_INTERNAL_RX_SW_BUFFER_CONST) */

#if(MAX7219_INTERNAL_TX_SW_BUFFER_CONST)
    volatile uint32 MAX7219_txBufferHead;
    volatile uint32 MAX7219_txBufferTail;
#endif /* (MAX7219_INTERNAL_TX_SW_BUFFER_CONST) */

#if(MAX7219_INTERNAL_RX_SW_BUFFER)
    /* Add one element to the buffer to receive full packet. One byte in receive buffer is always empty */
    volatile uint8 MAX7219_rxBufferInternal[MAX7219_RX_BUFFER_SIZE];
#endif /* (MAX7219_INTERNAL_RX_SW_BUFFER) */

#if(MAX7219_INTERNAL_TX_SW_BUFFER)
    volatile uint8 MAX7219_txBufferInternal[MAX7219_TX_BUFFER_SIZE];
#endif /* (MAX7219_INTERNAL_TX_SW_BUFFER) */


#if(MAX7219_RX_DIRECTION)

    /*******************************************************************************
    * Function Name: MAX7219_SpiUartReadRxData
    ********************************************************************************
    *
    * Summary:
    *  Retrieves the next data element from the receive buffer.
    *   - RX software buffer is disabled: Returns data element retrieved from
    *     RX FIFO. Undefined data will be returned if the RX FIFO is empty.
    *   - RX software buffer is enabled: Returns data element from the software
    *     receive buffer. Zero value is returned if the software receive buffer
    *     is empty.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Next data element from the receive buffer.
    *
    * Global Variables:
    *  Look into MAX7219_SpiInit for description.
    *
    *******************************************************************************/
    uint32 MAX7219_SpiUartReadRxData(void)
    {
        uint32 rxData = 0u;

        #if(MAX7219_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (MAX7219_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_RX_SW_BUFFER)
        {
            if(MAX7219_rxBufferHead != MAX7219_rxBufferTail)
            {
                /* There is data in RX software buffer */

                /* Calculate index to read from */
                locTail = (MAX7219_rxBufferTail + 1u);

                if(MAX7219_RX_BUFFER_SIZE == locTail)
                {
                    locTail = 0u;
                }

                /* Get data from RX software buffer */
                rxData = MAX7219_GetWordFromRxBuffer(locTail);

                /* Change index in the buffer */
                MAX7219_rxBufferTail = locTail;
            }
        }
        #else
        {
            rxData = MAX7219_RX_FIFO_RD_REG; /* Read data from RX FIFO */
        }
        #endif

        return(rxData);
    }


    /*******************************************************************************
    * Function Name: MAX7219_SpiUartGetRxBufferSize
    ********************************************************************************
    *
    * Summary:
    *  Returns the number of received data elements in the receive buffer.
    *   - RX software buffer disabled: returns the number of used entries in
    *     RX FIFO.
    *   - RX software buffer enabled: returns the number of elements which were
    *     placed in the receive buffer. This does not include the hardware RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of received data elements
    *
    *******************************************************************************/
    uint32 MAX7219_SpiUartGetRxBufferSize(void)
    {
        uint32 size;
        #if(MAX7219_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 locHead;
        #endif /* (MAX7219_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_RX_SW_BUFFER)
        {
            locHead = MAX7219_rxBufferHead;

            if(locHead >= MAX7219_rxBufferTail)
            {
                size = (locHead - MAX7219_rxBufferTail);
            }
            else
            {
                size = (locHead + (MAX7219_RX_BUFFER_SIZE - MAX7219_rxBufferTail));
            }
        }
        #else
        {
            size = MAX7219_GET_RX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: MAX7219_SpiUartClearRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the receive buffer and RX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_SpiUartClearRxBuffer(void)
    {
        #if(MAX7219_INTERNAL_RX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (MAX7219_INTERNAL_RX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_RX_SW_BUFFER)
        {
            intSourceMask = MAX7219_SpiUartDisableIntRx();

            MAX7219_CLEAR_RX_FIFO;

            /* Flush RX software buffer */
            MAX7219_rxBufferHead     = MAX7219_rxBufferTail;
            MAX7219_rxBufferOverflow = 0u;

            /* End RX transfer */
            MAX7219_ClearRxInterruptSource(MAX7219_INTR_RX_ALL);

            MAX7219_SpiUartEnableIntRx(intSourceMask);
        }
        #else
        {
            MAX7219_CLEAR_RX_FIFO;
        }
        #endif
    }

#endif /* (MAX7219_RX_DIRECTION) */


#if(MAX7219_TX_DIRECTION)

    /*******************************************************************************
    * Function Name: MAX7219_SpiUartWriteTxData
    ********************************************************************************
    *
    * Summary:
    *  Places a data entry into the transmit buffer to be sent at the next available
    *  bus time.
    *  This function is blocking and waits until there is space available to put the
    *  requested data in the transmit buffer.
    *
    * Parameters:
    *  txDataByte: the data to be transmitted.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_SpiUartWriteTxData(uint32 txData)
    {
        #if(MAX7219_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locHead;
            uint32 intSourceMask;
        #endif /* (MAX7219_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_TX_SW_BUFFER)
        {
            /* Head index to put data */
            locHead = (MAX7219_txBufferHead + 1u);

            /* Adjust TX software buffer index */
            if(MAX7219_TX_BUFFER_SIZE == locHead)
            {
                locHead = 0u;
            }

            while(locHead == MAX7219_txBufferTail)
            {
                /* Wait for space in TX software buffer */
            }

            /* TX software buffer has at least one room */

            if((MAX7219_txBufferHead == MAX7219_txBufferTail) &&
               (MAX7219_SPI_UART_FIFO_SIZE != MAX7219_GET_TX_FIFO_ENTRIES))
            {
                /* TX software buffer is empty: put data directly in TX FIFO */
                MAX7219_TX_FIFO_WR_REG = txData;
            }
            /* Put data in TX software buffer */
            else
            {
                /* Clear old status of INTR_TX_NOT_FULL. It sets at the end of transfer when TX FIFO is empty. */
                MAX7219_ClearTxInterruptSource(MAX7219_INTR_TX_NOT_FULL);

                MAX7219_PutWordInTxBuffer(locHead, txData);

                MAX7219_txBufferHead = locHead;

                /* Enable interrupt to transmit */
                intSourceMask  = MAX7219_INTR_TX_NOT_FULL;
                intSourceMask |= MAX7219_GetTxInterruptMode();
                MAX7219_SpiUartEnableIntTx(intSourceMask);
            }
        }
        #else
        {
            while(MAX7219_SPI_UART_FIFO_SIZE == MAX7219_GET_TX_FIFO_ENTRIES)
            {
                /* Block while TX FIFO is FULL */
            }

            MAX7219_TX_FIFO_WR_REG = txData;
        }
        #endif
    }


    /*******************************************************************************
    * Function Name: MAX7219_SpiUartPutArray
    ********************************************************************************
    *
    * Summary:
    *  Places an array of data into the transmit buffer to be sent.
    *  This function is blocking and waits until there is a space available to put
    *  all the requested data in the transmit buffer. The array size can be greater
    *  than transmit buffer size.
    *
    * Parameters:
    *  wrBuf:  pointer to an array with data to be placed in transmit buffer.
    *  count:  number of data elements to be placed in the transmit buffer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_SpiUartPutArray(const uint8 wrBuf[], uint32 count)
    {
        uint32 i;

        for(i=0u; i < count; i++)
        {
            MAX7219_SpiUartWriteTxData((uint32) wrBuf[i]);
        }
    }


    /*******************************************************************************
    * Function Name: MAX7219_SpiUartGetTxBufferSize
    ********************************************************************************
    *
    * Summary:
    * Returns the number of elements currently in the transmit buffer.
    *  - TX software buffer is disabled: returns the number of used entries in
    *    TX FIFO.
    *  - TX software buffer is enabled: returns the number of elements currently
    *    used in the transmit buffer. This number does not include used entries in
    *    the TX FIFO. The transmit buffer size is zero until the TX FIFO is
    *    not full.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Number of data elements ready to transmit.
    *
    *******************************************************************************/
    uint32 MAX7219_SpiUartGetTxBufferSize(void)
    {
        uint32 size;
        #if(MAX7219_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 locTail;
        #endif /* (MAX7219_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_TX_SW_BUFFER)
        {
            /* Get current Tail index */
            locTail = MAX7219_txBufferTail;

            if(MAX7219_txBufferHead >= locTail)
            {
                size = (MAX7219_txBufferHead - locTail);
            }
            else
            {
                size = (MAX7219_txBufferHead + (MAX7219_TX_BUFFER_SIZE - locTail));
            }
        }
        #else
        {
            size = MAX7219_GET_TX_FIFO_ENTRIES;
        }
        #endif

        return(size);
    }


    /*******************************************************************************
    * Function Name: MAX7219_SpiUartClearTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Clears the transmit buffer and TX FIFO.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_SpiUartClearTxBuffer(void)
    {
        #if(MAX7219_INTERNAL_TX_SW_BUFFER_CONST)
            uint32 intSourceMask;
        #endif /* (MAX7219_INTERNAL_TX_SW_BUFFER_CONST) */

        #if(MAX7219_CHECK_TX_SW_BUFFER)
        {
            intSourceMask = MAX7219_SpiUartDisableIntTx();

            MAX7219_CLEAR_TX_FIFO;

            /* Flush TX software buffer */
            MAX7219_txBufferHead = MAX7219_txBufferTail;

            /* End TX transfer if it is in progress */
            intSourceMask &= (uint32) ~MAX7219_INTR_TX_NOT_FULL;

            MAX7219_SpiUartEnableIntTx(intSourceMask);
        }
        #else
        {
            MAX7219_CLEAR_TX_FIFO;
        }
        #endif
    }

#endif /* (MAX7219_TX_DIRECTION) */


/*******************************************************************************
* Function Name: MAX7219_SpiUartDisableIntRx
********************************************************************************
*
* Summary:
*  Disables the RX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns the RX interrupt sources enabled before the function call.
*
*******************************************************************************/
uint32 MAX7219_SpiUartDisableIntRx(void)
{
    uint32 intSource;

    intSource = MAX7219_GetRxInterruptMode();

    MAX7219_SetRxInterruptMode(MAX7219_NO_INTR_SOURCES);

    return(intSource);
}


/*******************************************************************************
* Function Name: MAX7219_SpiUartDisableIntTx
********************************************************************************
*
* Summary:
*  Disables TX interrupt sources.
*
* Parameters:
*  None
*
* Return:
*  Returns TX interrupt sources enabled before function call.
*
*******************************************************************************/
uint32 MAX7219_SpiUartDisableIntTx(void)
{
    uint32 intSourceMask;

    intSourceMask = MAX7219_GetTxInterruptMode();

    MAX7219_SetTxInterruptMode(MAX7219_NO_INTR_SOURCES);

    return(intSourceMask);
}


#if(MAX7219_SCB_MODE_UNCONFIG_CONST_CFG)
    /*******************************************************************************
    * Function Name: MAX7219_PutWordInRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores a byte/word into the RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  index:      index to store data byte/word in the RX buffer.
    *  rxDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_PutWordInRxBuffer(uint32 idx, uint32 rxDataByte)
    {
        /* Put data in buffer */
        if(MAX7219_ONE_BYTE_WIDTH == MAX7219_rxDataBits)
        {
            MAX7219_rxBuffer[idx] = ((uint8) rxDataByte);
        }
        else
        {
            MAX7219_rxBuffer[(uint32)(idx << 1u)]      = LO8(LO16(rxDataByte));
            MAX7219_rxBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(rxDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: MAX7219_GetWordFromRxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from RX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  Returns byte/word read from RX buffer.
    *
    *******************************************************************************/
    uint32 MAX7219_GetWordFromRxBuffer(uint32 idx)
    {
        uint32 value;

        if(MAX7219_ONE_BYTE_WIDTH == MAX7219_rxDataBits)
        {
            value = MAX7219_rxBuffer[idx];
        }
        else
        {
            value  = (uint32) MAX7219_rxBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32)MAX7219_rxBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }


    /*******************************************************************************
    * Function Name: MAX7219_PutWordInTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Stores byte/word into the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx:        index to store data byte/word in the TX buffer.
    *  txDataByte: byte/word to store.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void MAX7219_PutWordInTxBuffer(uint32 idx, uint32 txDataByte)
    {
        /* Put data in buffer */
        if(MAX7219_ONE_BYTE_WIDTH == MAX7219_txDataBits)
        {
            MAX7219_txBuffer[idx] = ((uint8) txDataByte);
        }
        else
        {
            MAX7219_txBuffer[(uint32)(idx << 1u)]      = LO8(LO16(txDataByte));
            MAX7219_txBuffer[(uint32)(idx << 1u) + 1u] = HI8(LO16(txDataByte));
        }
    }


    /*******************************************************************************
    * Function Name: MAX7219_GetWordFromTxBuffer
    ********************************************************************************
    *
    * Summary:
    *  Reads byte/word from the TX buffer.
    *  Only available in the Unconfigured operation mode.
    *
    * Parameters:
    *  idx: index to get data byte/word from the TX buffer.
    *
    * Return:
    *  Returns byte/word read from the TX buffer.
    *
    *******************************************************************************/
    uint32 MAX7219_GetWordFromTxBuffer(uint32 idx)
    {
        uint32 value;

        if(MAX7219_ONE_BYTE_WIDTH == MAX7219_txDataBits)
        {
            value = (uint32) MAX7219_txBuffer[idx];
        }
        else
        {
            value  = (uint32) MAX7219_txBuffer[(uint32)(idx << 1u)];
            value |= (uint32) ((uint32) MAX7219_txBuffer[(uint32)(idx << 1u) + 1u] << 8u);
        }

        return(value);
    }

#endif /* (MAX7219_SCB_MODE_UNCONFIG_CONST_CFG) */


/* [] END OF FILE */
